/*
 * http set random variable module.
 *
 * THIS SOFTWARE IS UNDER MIT LICENSE.
 * Copyright (C) 2008 Guy Naor - Morph Labs (guy@mor.ph)
 *
 * Read LICENSE file for more informations.
 */


#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_http.h>

static char *ngx_http_random_set_random(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);

static ngx_int_t ngx_http_random_variable(ngx_http_request_t *r, ngx_http_variable_value_t *v, uintptr_t data); 
static ngx_int_t ngx_http_random_set_variable(ngx_http_request_t *r, ngx_http_variable_value_t *v, uintptr_t data);


static ngx_command_t  ngx_http_set_random_commands[] = {

    { ngx_string("set_random"),
      NGX_HTTP_SRV_CONF|NGX_HTTP_SIF_CONF|NGX_HTTP_LOC_CONF|NGX_HTTP_LIF_CONF|NGX_CONF_TAKE1,
      ngx_http_random_set_random,
      0, 
      0,
      NULL },

      ngx_null_command
};


static ngx_http_module_t  ngx_http_set_random_module_ctx = {
    NULL,  NULL, NULL, NULL,  NULL, NULL, NULL, NULL 
};


ngx_module_t  ngx_http_set_random_module = {
    NGX_MODULE_V1,
    &ngx_http_set_random_module_ctx,       /* module context */
    ngx_http_set_random_commands,          /* module directives */
    NGX_HTTP_MODULE,                       /* module type */
    NULL,                                  /* init master */
    NULL,                                  /* init module */
    NULL,                                  /* init process */
    NULL,                                  /* init thread */
    NULL,                                  /* exit thread */
    NULL,                                  /* exit process */
    NULL,                                  /* exit master */
    NGX_MODULE_V1_PADDING
};


static char * ngx_http_random_set_random(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
{
    ngx_str_t           *value;
    ngx_http_variable_t *v;

    value = cf->args->elts;

    if (value[1].data[0] != '$') {
        ngx_conf_log_error(NGX_LOG_EMERG, cf, 0, "invalid variable name \"%V\"", &value[1]);
        return NGX_CONF_ERROR;
    }

    value[1].len--;
    value[1].data++;

    v = ngx_http_add_variable(cf, &value[1], 0); 
    if (v == NULL) {
        return NGX_CONF_ERROR;
    }

    if (v->get_handler == NULL )
    {
        v->get_handler = ngx_http_random_variable;
    } else { 
        ngx_conf_log_error(NGX_LOG_EMERG, cf, 0, "variable already defined as a non-random variable: \"%V\"", &value[1]);
        return NGX_CONF_ERROR;
    }

    return NGX_CONF_OK;
}


static ngx_int_t ngx_http_random_variable(ngx_http_request_t *r, ngx_http_variable_value_t *v, uintptr_t data) 
{
  v->data = ngx_palloc(r->pool, 32);
  sprintf((char *)v->data, "%i", rand()); 
  v->len  = ngx_strlen(v->data);

  /* Set all required params */
  v->valid = 1;
  v->no_cacheable = 0;
  v->not_found = 0;

  return NGX_OK;
}



